const drawing = document.getElementById("canvas");
const width = drawing.width;
const height = drawing.height;
const u = 1.66054e-27;
const m = 28 * u;
const k = 1.38065e-23;
const ao = 5.29177e-11;
const rad_rate = 5;
const vel_rate = 400;
const animateButton = document.getElementById("startButton1");
const restartButton = document.getElementById("restartButton");
const play = document.getElementById('play');
const numButton = document.getElementById("num1");
const radiusButton = document.getElementById('radius');
const velocityButton = document.getElementById("velocity");
const tempButton = document.getElementById("temp");
const showNum = document.getElementById("show-num");
const showTemp = document.getElementById("show-temp");
const showPressure = document.getElementById("show-pressure");
const showPressure3 = document.getElementById("show-pressure-3");
const showMeanVel = document.getElementById("show-aver-vel");
const showRMSVel = document.getElementById("show-rms-vel");
const showLambda = document.getElementById("show-lambda");
const chart_velocity = echarts.init(document.getElementById('chartContainer'), null, {renderer: 'svg'});
const chart_free_path = echarts.init(document.getElementById('chartContainer1'), null, {renderer: 'svg'});

let num = numButton.value;
let radius = radiusButton.value * rad_rate;
let velocity = velocityButton.value / vel_rate;
let temp = parseFloat(tempButton.value);
let param = 2 * k * temp / m;
let mean_free_path_data = [];
const balls = Array(parseInt(num));
const context = drawing.getContext("2d");

let count = 0;
let mode = true;
let state = false;
let maxVelocity = 4;
let free_path = 0;
let collision_num = 0;

window.onresize = function () {
    chart_velocity.resize();
    chart_free_path.resize();
}

function changeMode(obj) {
    mode = eval(obj.children[0].value);
    if (mode) {
        tempButton.readOnly = true;
        velocityButton.readOnly = false;
    } else {
        tempButton.readOnly = false;
        velocityButton.readOnly = true;
    }
}

function initMaxwell() {
    let index = 0;
    let data = Array(num);
    while (index < num) {
        let v2 = param * Math.random();
        let F = Math.random() / param;
        if (F < Math.exp(-v2 / param) / param) {
            data[index] = v2;
            index += 1;
        }
    }
    let sum = data.reduce(function (sum, item) {
        return sum + item;
    }, 0);
    return {
        sum: sum,
        data: data
    };
}

animateButton.onclick = function () {
    if (!state) {
        state = true;
        play.classList.replace('fa-play', 'fa-pause');
    } else {
        state = false;
        play.classList.replace('fa-pause', 'fa-play');
    }
}

restartButton.onclick = function () {
    state = false;
    play.classList.replace('fa-pause', 'fa-play');
    mean_free_path_data = [];
    free_path = 0;
    collision_num = 0;
    count = 0;
    init();
    init_table();
}

numButton.onchange = function () {
    num = parseInt(numButton.value);
    mean_free_path_data = [];
    free_path = 0;
    collision_num = 0;
    if (num > balls.length) {
        if (mode) {
            for (let i = balls.length; i < num; i++) {
                balls[i] = new Ball(velocity);
            }
        } else {
            let obj = initMaxwell();
            for (let i = 0; i < num; i++) {
                balls[i] = new Ball(Math.sqrt(obj.data[i] / obj.sum * num * param) / vel_rate);
            }
        }
    } else {
        balls.splice(num, balls.length - num);
        if (mode) {
            for (let i = 0; i < num; i++) {
                balls[i] = new Ball(velocity);
            }
        } else {
            let obj = initMaxwell();
            for (let i = 0; i < num; i++) {
                balls[i] = new Ball(Math.sqrt(obj.data[i] / obj.sum * num * param) / vel_rate);
            }
        }
    }
}

radiusButton.onchange = function () {
    radius = radiusButton.value * rad_rate;
    mean_free_path_data = [];
    free_path = 0;
    collision_num = 0;
    balls.forEach(function (ball) {
        ball.r = radius;
    })
}

velocityButton.onchange = function () {
    velocity = velocityButton.value / vel_rate;
    mean_free_path_data = [];
    free_path = 0;
    collision_num = 0;
    balls.forEach(function (ball) {
        let vx = ball.vx / ball.velocity * velocity;
        let vy = ball.vy / ball.velocity * velocity;
        ball.vx = vx;
        ball.vy = vy;
    })
}

tempButton.onchange = function () {
    mean_free_path_data = [];
    free_path = 0;
    collision_num = 0;
    balls.forEach(function (ball) {
        ball.vx = Math.sign(ball.vx) * Math.sqrt(Math.pow(ball.vx, 2) / temp * parseFloat(tempButton.value));
        ball.vy = Math.sign(ball.vy) * Math.sqrt(Math.pow(ball.vy, 2) / temp * parseFloat(tempButton.value));
    })
    temp = parseFloat(tempButton.value);
    param = 2 * k * temp / m;
}

// noinspection DuplicatedCode
class Ball {
    constructor(v) {
        let direction = 2 * Math.PI * Math.random();
        this.x = width * Math.random();
        this.y = height * Math.random();
        this.vx = v * Math.cos(direction);
        this.vy = v * Math.sin(direction);
        this.r = radius;
        this.free_path = 0;
    }

    get velocity() {
        return Math.sqrt(Math.pow(this.vx, 2) + Math.pow(this.vy, 2));
    }

    move() {
        if (0 < this.x + this.vx && this.x + this.vx < width) {
            this.x += this.vx;
        } else {
            if (this.x + this.vx <= 0) {
                this.x = -this.x - this.vx;
            } else {
                this.x = 2 * width - this.x - this.vx;
            }
            this.vx = -this.vx;
        }

        if (0 < this.y + this.vy && this.y + this.vy < height) {
            this.y += this.vy;
        } else {
            if (this.y + this.vy <= 0) {
                this.y = -this.y - this.vy;
            } else {
                this.y = 2 * height - this.y - this.vy;
            }
            this.vy = -this.vy;
        }
        this.free_path += this.velocity * ao / radius * 1e10;
    }

    distance(that) {
        return Math.sqrt((this.x - that.x) * (this.x - that.x) + (this.y - that.y) * (this.y - that.y));
    }

    toward(that) {
        return (this.vx - that.vx) * (this.x - that.x) + (this.vy - that.vy) * (this.y - that.y);
    }

    collision(that) {
        collision_num += 2;
        free_path += this.free_path + that.free_path;
        this.free_path = 0;
        that.free_path = 0;
        let factor = (this.toward(that)) /
            Math.pow(this.distance(that), 2);
        this.vx = this.vx - factor * (this.x - that.x);
        this.vy = this.vy - factor * (this.y - that.y);
        that.vx = that.vx + factor * (this.x - that.x);
        that.vy = that.vy + factor * (this.y - that.y);
    }

    draw() {
        context.save();

        context.beginPath();
        const grad = context.createRadialGradient(this.x, this.y, 0, this.x, this.y, this.r * 1.5);
        if (!maxVelocity) {
            grad.addColorStop(0, 'yellow');
        } else {
            grad.addColorStop(0, "rgba(255, " + 255 * (1 - this.velocity / maxVelocity) + ", 0, 0.8)");
        }
        grad.addColorStop(1, "white");
        context.fillStyle = grad;
        context.arc(this.x, this.y, this.r, 0, 2 * Math.PI);
        context.fill();
        context.closePath();

        context.restore();
    }
}

function init() {
    context.clearRect(0, 0, width, height);
    if (mode) {
        for (let index = 0; index < num; index++) {
            balls[index] = new Ball(velocity);
        }
    } else {
        let obj = initMaxwell();
        for (let i = 0; i < num; i++) {
            balls[i] = new Ball(Math.sqrt(obj.data[i] / obj.sum * num * param) / vel_rate);
        }
    }
}

function executeBalls() {
    for (let i = 0; i < num; i++) {
        balls[i].move();
        for (let j = i + 1; j < num; j++) {
            if (balls[i].distance(balls[j]) <= balls[i].r + balls[j].r && balls[i].toward(balls[j]) < 0) {
                balls[i].collision(balls[j]);
            }
        }
        balls[i].draw()
    }
}

function collectVelocities() {
    let velocities = Array(20);
    for (let i = 0; i < 20; i++) {
        let min = (i * maxVelocity / 20).toFixed(2);
        let max = ((i + 1) * maxVelocity / 20).toFixed(2);
        velocities[i] = [min * vel_rate, balls.reduce(function (sum, ball) {
            return (ball.velocity > min && ball.velocity <= max) ? sum + 1 : sum;
        }, 0), max * vel_rate];
    }
    return velocities;
}

function getSum() {
    return balls.reduce(function (sum, ball) {
        return sum + ball.velocity * vel_rate;
    }, 0);
}

function getSquareSum() {
    return balls.reduce(function (sum, ball) {
        return sum + Math.pow(ball.velocity * vel_rate, 2);
    }, 0);
}

function showInfo() {
    let rms = Math.sqrt(getSquareSum() / num);
    let mean = getSum() / num;
    let T = m * rms * rms / (2 * k);
    let p = num * k * T / (width * height) * Math.pow(rad_rate / ao, 2);
    let p3 = k * T * Math.pow(num / (width * height) * Math.pow(rad_rate / ao, 2), 3 / 2);
    let lambda = free_path / collision_num;
    showNum.innerText = num;
    showTemp.innerText = T.toFixed(3);
    showPressure.innerText = (p * 1000).toFixed(3);
    showMeanVel.innerText = mean.toFixed(3);
    showPressure3.innerText = (p3 / 1000).toFixed(3);
    showRMSVel.innerText = rms.toFixed(3);
    showLambda.innerText = lambda.toFixed(3);
    if (mode) {
        temp = T;
        tempButton.value = T.toFixed(0);
        param = 2 * k * temp / m;
    } else {
        velocityButton.value = rms.toFixed(0);
    }
}

function run() {
    requestAnimationFrame(run);

    if (state) {
        if (count === 0) {
            init();
        }
        context.clearRect(0, 0, width, height);
        maxVelocity = balls.reduce(function (max, item) {
            return max > item.velocity ? max : item.velocity;
        })
        executeBalls();
        if (count % 60 === 0) {
            drawTable();
        }
        if (count % 60 === 30) {
            showInfo();
        }
        count += 1;
    }
}

function renderItem(params, api) {
    // 这个根据自己的需求适当调节
    const yValue = api.value(1);
    const start = api.coord([api.value(0), yValue]);
    const size = api.size([api.value(2) - api.value(0), yValue]);
    const style = api.style();

    return {
        // 矩形及配置
        type: 'rect',
        shape: {
            x: start[0] + 1,
            y: start[1],
            width: size[0] - 2,
            height: size[1]
        },
        style: style
    };
}

function init_free_path_table() {
    const option = {
        title: {
            text: '平均自由程'
        },
        tooltip: {
            trigger: 'axis',
        },
        legend: {
            data: ['mean free path'],
            formatter: (_ => "平均自由程")
        },
        xAxis: {
            name: '碰撞次数',
            nameLocation: 'middle',
            type: 'value',
            nameTextStyle: {
                padding: [15, 0, 0, 0],
                fontSize: 15
            },
            min: 0,
            max: 'dataMax'
        },
        yAxis: {
            name: '平均自由程(Å)',
            nameLocation: 'middle',
            nameTextStyle: {
                padding: [0, 0, 15, 0],
                fontSize: 15
            },
            type: 'value',
            min: 0,
            max: 1
        },
        series: [{
            name: 'mean free path',
            type: 'line',
            data: []

        }]
    };

    chart_free_path.setOption(option);
}

function init_velocity_table() {

    const option_velocity = {
        title: {
            text: '速率分布'
        },
        tooltip: {},
        legend: {
            data: ['particles'],
            formatter: (_ => "分子数")
        },
        xAxis: {
            name: '速率(m/s)',
            nameTextStyle: {
                padding: [15, 0, 0, 0],
                fontSize: 15
            },
            nameLocation: 'middle',
            type: 'value',
            max: Math.ceil(maxVelocity * vel_rate)
        },
        yAxis: {
            name: '分子数',
            nameTextStyle: {
                padding: [0, 0, 15, 0],
                fontSize: 15
            },
            nameLocation: 'middle',
            type: 'value',
            min: 0,
            max: Math.ceil(num / 2 / 10) * 10
        },
        series: [{
            name: 'particles',
            type: 'custom',
            renderItem: renderItem,
            data: [0, 0, 0]
        }]
    };

    chart_velocity.setOption(option_velocity);
}

function init_table() {
    init_free_path_table();
    init_velocity_table();
}

function drawTable() {
    drawTable_velocity();
    drawTable_free_path();
}

function drawTable_free_path() {
    if (mean_free_path_data.length === 0 || collision_num > mean_free_path_data[mean_free_path_data.length - 1][0]) {
        mean_free_path_data.push([collision_num.toFixed(0), (free_path / collision_num).toFixed(2)]);
    }
    if (mean_free_path_data.length > 20) {
        mean_free_path_data.shift();
    }
    const option = {
        title: {
            text: '平均自由程'
        },
        tooltip: {
            trigger: 'axis',
        },
        legend: {
            data: ['平均自由程'],
        },
        xAxis: {
            name: '碰撞次数',
            nameLocation: 'middle',
            type: 'value',
            nameTextStyle: {
                padding: [15, 0, 0, 0],
                fontSize: 15
            },
            min: 'dataMin',
            max: 'dataMax'
        },
        yAxis: {
            name: '平均自由程(Å)',
            nameLocation: 'middle',
            nameTextStyle: {
                padding: [0, 0, 15, 0],
                fontSize: 15
            },
            type: 'value',
            min: 0,
            max: Math.ceil(free_path / collision_num * 1.3)
        },
        series: [{
            name: '平均自由程',
            type: 'line',
            data: mean_free_path_data,

        }]
    };
    // chart_free_path.clear();
    chart_free_path.setOption(option);
}

function drawTable_velocity() {
    let velocities = collectVelocities();
    let max = velocities.reduce(function (max, velocity) {
        return velocity[1] < max ? max : velocity[1];
    }, 0);
    const option_velocity = {
        title: {
            text: '速率分布'
        },
        tooltip: {
            trigger: 'axis',
            // formatter: '{a}:{c}'
        },
        legend: {
            data: ['分子数'],
        },
        xAxis: {
            name: '速率(m/s)',
            nameTextStyle: {
                padding: [15, 0, 0, 0],
                fontSize: 15
            },
            nameLocation: 'middle',
            type: 'value',
            min: 0,
            max: Math.ceil(maxVelocity * vel_rate)
        },
        yAxis: {
            name: '分子数',
            nameTextStyle: {
                padding: [0, 0, 15, 0],
                fontSize: 15
            },
            nameLocation: 'middle',
            type: 'value',
            min: 0,
            max: Math.ceil(max / 10) * 10
        },
        series: [{
            name: '分子数',
            type: 'custom',
            renderItem: renderItem,
            data: velocities
        }]
    };
    chart_velocity.clear();
    chart_velocity.setOption(option_velocity);
}

$(function () {
    $('[data-toggle="tooltip"]').tooltip()
})

if (drawing.getContext) {
    init_table();
    run();
} else {
    const support = document.getElementById("support-canvas");
    support.innerHTML = "The Explorer does not support canvas. ";
}
